// src/middleware/auth.ts
import { AppStrategy, createClient, OAuthStrategy } from "@wix/sdk";
import { WIX_CLIENT_ID as WIX_CLIENT_ID3 } from "astro:env/client";
import {
  WIX_CLIENT_INSTANCE_ID,
  WIX_CLIENT_PUBLIC_KEY,
  WIX_CLIENT_SECRET
} from "astro:env/server";

// src/utils/authStrategyAsyncLocalStorage.ts
import { AsyncLocalStorage } from "node:async_hooks";
var authStrategyAsyncLocalStorage = new AsyncLocalStorage();

// src/utils/getSessionTokensFromCookie.ts
import { TokenRole } from "@wix/sdk";
import { WIX_CLIENT_ID } from "astro:env/client";
import { z } from "astro/zod";
var tokensSchema = z.object({
  clientId: z.string(),
  tokens: z.object({
    accessToken: z.object({
      expiresAt: z.number(),
      value: z.string()
    }),
    refreshToken: z.object({
      role: z.nativeEnum(TokenRole),
      value: z.string()
    })
  })
});
function getSessionTokensFromCookie(context) {
  if (context.isPrerendered) {
    return;
  }
  const rawCookie = context.cookies.get("wixSession")?.json();
  if (rawCookie) {
    const tokensParseResult = tokensSchema.safeParse(rawCookie);
    if (tokensParseResult.success && tokensParseResult.data.clientId === WIX_CLIENT_ID) {
      return tokensParseResult.data;
    }
  }
}

// src/utils/saveSessionTokensToCookie.ts
import { WIX_CLIENT_ID as WIX_CLIENT_ID2 } from "astro:env/client";
function sessionCookieJson(tokens) {
  return {
    clientId: WIX_CLIENT_ID2,
    tokens
  };
}
function saveSessionTokensToCookie(context, tokens) {
  context.cookies.set("wixSession", sessionCookieJson(tokens), {
    path: "/",
    secure: true
  });
}

// src/middleware/auth.ts
var contextClient = createClient({
  auth: {
    async getAuthHeaders(host) {
      const store = authStrategyAsyncLocalStorage.getStore();
      if (!store?.auth) {
        throw new Error(
          "No authentication strategy found in the current context"
        );
      }
      return store.auth.getAuthHeaders(host);
    }
  }
});
var elevatedContextClient = createClient({
  auth: {
    async getAuthHeaders(host) {
      const store = authStrategyAsyncLocalStorage.getStore();
      if (!store?.elevatedAuth) {
        throw new Error(
          "No elevated authentication strategy found in the current context"
        );
      }
      return store.elevatedAuth.getAuthHeaders(host);
    }
  }
});
contextClient.enableContext("global");
elevatedContextClient.enableContext("global", { elevated: true });
var onRequest = async (context, next) => {
  const sessionTokensFromCookie = getSessionTokensFromCookie(context);
  const auth = OAuthStrategy({
    clientId: WIX_CLIENT_ID3
  });
  if (sessionTokensFromCookie) {
    auth.setTokens(sessionTokensFromCookie.tokens);
  } else {
    auth.setTokens(await auth.generateVisitorTokens());
  }
  const elevatedAuth = AppStrategy({
    appId: WIX_CLIENT_ID3,
    appSecret: WIX_CLIENT_SECRET,
    instanceId: WIX_CLIENT_INSTANCE_ID,
    publicKey: WIX_CLIENT_PUBLIC_KEY
  });
  const response = await authStrategyAsyncLocalStorage.run(
    {
      auth,
      elevatedAuth
    },
    () => next()
  );
  if (!context.isPrerendered && sessionTokensFromCookie?.tokens.accessToken.expiresAt !== auth.getTokens().accessToken.expiresAt) {
    saveSessionTokensToCookie(context, auth.getTokens());
  }
  return response;
};
export {
  onRequest
};
