import type { MiddlewareHandler } from 'astro';
import { AppStrategy, createClient, OAuthStrategy } from '@wix/sdk';
import { WIX_CLIENT_ID } from 'astro:env/client';
import {
  WIX_CLIENT_INSTANCE_ID,
  WIX_CLIENT_PUBLIC_KEY,
  WIX_CLIENT_SECRET,
} from 'astro:env/server';
import { authStrategyAsyncLocalStorage } from '../utils/authStrategyAsyncLocalStorage.js';
import { getSessionTokensFromCookie } from '../utils/getSessionTokensFromCookie.js';
import { saveSessionTokensToCookie } from '../utils/saveSessionTokensToCookie.js';

const contextClient = createClient({
  auth: {
    async getAuthHeaders(host) {
      const store = authStrategyAsyncLocalStorage.getStore();

      if (!store?.auth) {
        throw new Error(
          'No authentication strategy found in the current context'
        );
      }

      return store.auth.getAuthHeaders(host);
    },
  },
});

const elevatedContextClient = createClient({
  auth: {
    async getAuthHeaders(host) {
      const store = authStrategyAsyncLocalStorage.getStore();

      if (!store?.elevatedAuth) {
        throw new Error(
          'No elevated authentication strategy found in the current context'
        );
      }

      return store.elevatedAuth.getAuthHeaders(host);
    },
  },
});

contextClient.enableContext('global');
elevatedContextClient.enableContext('global', { elevated: true });

export const onRequest: MiddlewareHandler = async (context, next) => {
  const sessionTokensFromCookie = getSessionTokensFromCookie(context);

  const auth = OAuthStrategy({
    clientId: WIX_CLIENT_ID,
  });

  if (sessionTokensFromCookie) {
    auth.setTokens(sessionTokensFromCookie.tokens);
  } else {
    auth.setTokens(await auth.generateVisitorTokens());
  }

  const elevatedAuth = AppStrategy({
    appId: WIX_CLIENT_ID,
    appSecret: WIX_CLIENT_SECRET,
    instanceId: WIX_CLIENT_INSTANCE_ID,
    publicKey: WIX_CLIENT_PUBLIC_KEY,
  });

  const response = await authStrategyAsyncLocalStorage.run(
    {
      auth,
      elevatedAuth,
    },
    () => next()
  );

  if (
    !context.isPrerendered &&
    sessionTokensFromCookie?.tokens.accessToken.expiresAt !==
      auth.getTokens().accessToken.expiresAt
  ) {
    saveSessionTokensToCookie(context, auth.getTokens());
  }

  return response;
};
