import { join } from 'node:path';
import { cwd } from 'node:process';
import type { AstroIntegrationLogger } from 'astro';
import { globby } from 'globby';
import { outdent } from 'outdent';
import type { DevCenterComponent } from '../schemas.js';
import type { Model } from '../types.js';
import { Component, UnknownComponent } from '../components.js';
import { EXTENSIONS_DIR } from '../directories.js';
import {
  baseDevCenterComponentSchema,
  devCenterComponentSchema,
} from '../schemas.js';
import { pathExists, readJson } from './fs-utils.js';
import { loadEnvVars } from './loadEnvVars.js';

export async function createProjectModel(logger: AstroIntegrationLogger) {
  const rootDir = cwd();
  const { appId, env } = loadEnvVars(rootDir, logger);

  const componentsPaths = await globby(
    pathToGlobby(join(EXTENSIONS_DIR, '*')),
    {
      cwd: rootDir,
      onlyDirectories: true,
    }
  );

  const components: Component[] = [];
  const unknownComponents: UnknownComponent[] = [];

  for (const componentPath of componentsPaths) {
    const extensionManifest = await readExtensionManifest(componentPath);
    const knownManifest = await parseKnownExtensionManifest(extensionManifest);

    if (knownManifest != null) {
      components.push(createComponent(knownManifest, componentPath));
      continue;
    }

    const unknownManifest = await parseBaseExtensionManifest(extensionManifest);

    unknownComponents.push(
      UnknownComponent.Unknown({
        directory: componentPath,
        manifest: unknownManifest,
      })
    );
  }

  return {
    appId,
    components,
    env,
    rootDir,
    unknownComponents,
  } satisfies Model;
}

// https://github.com/mrmlnc/fast-glob/releases/tag/3.0.0
// globby using fast-glob under the hood accepts only forward slashes in patterns.
function pathToGlobby(path: string) {
  return path.replaceAll('\\', '/');
}

function createComponent(manifest: DevCenterComponent, directory: string) {
  switch (manifest.compType) {
    case 'BACK_OFFICE_EXTENSION_MENU_ITEM':
      return Component.BackofficeExtensionMenuPlugin({
        directory,
        manifest,
      });
    case 'BACK_OFFICE_EXTENSION_WIDGET': {
      return Component.BackofficeExtensionWidget({
        directory,
        manifest,
      });
    }
    case 'BACK_OFFICE_MODAL': {
      return Component.BackofficeModal({
        directory,
        manifest,
      });
    }
    case 'BACK_OFFICE_PAGE': {
      return Component.BackofficePage({
        directory,
        manifest,
      });
    }
    case 'ECOM_ADDITIONAL_FEES': {
      return Component.EcomAdditionalFees({
        directory,
        manifest,
      });
    }
    case 'ECOM_DISCOUNTS_TRIGGER': {
      return Component.EcomDiscountsTrigger({
        directory,
        manifest,
      });
    }
    case 'ECOM_PAYMENT_SETTINGS': {
      return Component.EcomPaymentSettings({
        directory,
        manifest,
      });
    }
    case 'ECOM_SHIPPING_RATES': {
      return Component.EcomShippingRates({
        directory,
        manifest,
      });
    }
    case 'ECOM_VALIDATIONS': {
      return Component.EcomValidations({
        directory,
        manifest,
      });
    }
    case 'GIFT_CARDS_PROVIDER': {
      return Component.EcomGiftCardsProvider({
        directory,
        manifest,
      });
    }
    case 'WEBHOOK': {
      return Component.Webhook({
        directory,
        manifest,
      });
    }
  }
}

async function parseKnownExtensionManifest(extensionManifest: unknown) {
  const appManifestResult =
    devCenterComponentSchema.safeParse(extensionManifest);

  if (appManifestResult.error) {
    return null;
  }

  return appManifestResult.data;
}

async function parseBaseExtensionManifest(extensionManifest: unknown) {
  const appManifestResult =
    baseDevCenterComponentSchema.safeParse(extensionManifest);

  if (!appManifestResult.success) {
    throw new Error(outdent`
      Invalid extension configuration:

      ${appManifestResult.error.errors
        .map((e) => `${e.path.join('.')}: ${e.message}`)
        .join('\n')}
    `);
  }

  return appManifestResult.data;
}

async function readExtensionManifest(directoryPath: string) {
  const manifestFilePath = join(directoryPath, 'extension.json');

  if (!(await pathExists(manifestFilePath))) {
    throw new Error('Missing extension manifest');
  }

  return await readJson(manifestFilePath);
}
